#ifndef ARRAY_SUBSCRIPT_PROCESSOR_HPP
#define ARRAY_SUBSCRIPT_PROCESSOR_HPP

#include <language/ast/ASTNode.hpp>
#include <language/node_processor/INodeProcessor.hpp>
#include <language/utils/ParseError.hpp>

template <typename ArrayTypeT>
class ArraySubscriptProcessor : public INodeProcessor
{
 private:
  ASTNode& m_array_subscript_expression;

 public:
  DataVariant
  execute(ExecutionPolicy& exec_policy)
  {
    auto& index_expression = *m_array_subscript_expression.children[1];

    const int64_t index_value = [&](DataVariant&& value_variant) -> int64_t {
      int64_t index_value = 0;
      std::visit(
        [&](auto&& value) {
          using ValueT = std::decay_t<decltype(value)>;
          if constexpr (std::is_integral_v<ValueT>) {
            index_value = value;
          } else {
            // LCOV_EXCL_START
            throw ParseError("unexpected error: invalid index type", std::vector{index_expression.begin()});
            // LCOV_EXCL_STOP
          }
        },
        value_variant);
      return index_value;
    }(index_expression.execute(exec_policy));

    auto& array_expression = *m_array_subscript_expression.children[0];

    auto&& array_value = array_expression.execute(exec_policy);
    ArrayTypeT& array  = std::get<ArrayTypeT>(array_value);

    return array[index_value];
  }

  ArraySubscriptProcessor(ASTNode& array_subscript_expression)
    : m_array_subscript_expression{array_subscript_expression}
  {}

  virtual ~ArraySubscriptProcessor() = default;
};

#endif   // ARRAY_SUBSCRIPT_PROCESSOR_HPP
